/***
   NAME
     Tritrophic.c
   PURPOSE

     Implement the tritrophic model (PNAS, 2002) with (1) a small invasion for
     the predator and possible size-selective harvesting of the consumer.

   Last Revision : AMdR - Dec 01, 2015
***/
#include  "escbox.h"


/*
 *===========================================================================
 *
 *		LABELLING ENVIRONMENT AND I-STATE VARIABLES
 *
 *===========================================================================
 */

#define time		env[0]
#define resource	env[1]
#define predators	env[2]

#define	length		i_state( 0)		// column  2

#define	IDage		i_const( 0)		// column  3
#define IDgrowth	i_const( 1)		// column  4
#define IDfecundity    	i_const( 2)		// column  5
#define IDpmort      	i_const( 3)		// column  6
#define IDsmort      	i_const( 4)		// column  7
#define IDnbegin      	i_const( 5)		// column  8



/*
 *===========================================================================
 *
 *		DEFINING AND LABELLING CONSTANTS AND PARAMETERS
 *
 *===========================================================================
 */

#define RHO		parameter[ 0]	// Default: 0.1
#define K		parameter[ 1]	// Default: 0.003

#define LB		parameter[ 2]	// Default: 7.0
#define LJ		parameter[ 3]	// Default: 110.0
#define LM		parameter[ 4]	// Default: 300.0

#define L1		parameter[ 5]	// Default: Lb
#define W		parameter[ 6]	// Default: 20.0

#define IM		parameter[ 7]	// Default: 1.0E-4
#define RH		parameter[ 8]	// Default: 1.5E-5

#define GAMMA		parameter[ 9]	// Default: 0.006
#define RM		parameter[10]	// Default: 0.003
#define MUC		parameter[11]	// Default: 0.01

#define	KAPPA		parameter[12]	// Default fraction to growth

#define MUS		parameter[13]	// Default: 0.2
#define QS		parameter[14]	// Default: 0.2

#define LWCONST		parameter[15]	// Default: 9.0E-6

#define ACP		parameter[16]	// Default: 10000.0
#define HCP		parameter[17]	// Default: 0.1
#define EPS		parameter[18]	// Default: 0.6
#define MUP		parameter[19]	// Default: 0.01

#define PINV		parameter[20]
#define MUHARV		parameter[21]
#define MINSIZEHARV	parameter[22]
#define HARVSTART	parameter[23]
#define HARVDUR		parameter[24]

#define LAKEVOLUME	parameter[25]

#define MIN_SURVIVAL	1.0E-9


/*
 *===========================================================================
 *
 * USER INITIALIZATION ROUTINE ALLOWS OPERATIONS ON INITAL POPULATIONS
 *
 *===========================================================================
 */

void	UserInit(int argc, char **argv, double *env, population *pop)
  
{
  int			i;

  switch (argc)
    {
    case 6:
      MUHARV    = atof(argv[5]);
    case 5:
      PINV      = atof(argv[4]);
    case 4:
      K    	= atof(argv[3]);
    case 3:
      MUP  	= atof(argv[2]);
    default:
      break;
    }

  for(i=0; i<cohort_no[0]; i++)
    if (ismissing(popIDcard[0][i][IDnbegin]))
      popIDcard[0][i][IDnbegin] = pop[0][i][number];

  return;
}




/*
 *===========================================================================
 *
 *	SPECIFICATION OF THE NUMBER AND VALUES OF BOUNDARY POINTS
 *
 *===========================================================================
 */

void	SetBpointNo(double *env, population *pop, int *bpoint_no)
  
{
  bpoint_no[0] = 1;
  
  return;
}




/*===========================================================================*/

void	SetBpoints(double *env, population *pop, population *bpoints)
  
{
  bpoints[0][0][length] = LB;
  
  return;
}




/*
 *===========================================================================
 *
 *			SPECIFICATION OF DERIVATIVES
 *
 *===========================================================================
 */

void	Gradient(double *env,     population *pop,     population *ofs,
		 double *envgrad, population *popgrad, population *ofsgrad,
		 population *bpoints)
  
{
  double		Fr, len, lmax, lstarve, lcrit;
  double		grazing, newborns, B;
  double		mort, predmort;
  register int		i;

  Fr       = resource/(RH+resource);
  lmax     = LM*Fr;
  lstarve  = lmax/KAPPA;
  lcrit    = lstarve - QS*(lstarve - lmax);
  
  grazing  = 0.0;
  newborns = 0.0;
  B        = 0.0;

  for(i=cohort_no[0]-1; i>=0; i--)
    {
      len = pop[0][i][length];

      if (len < L1) continue;
      if ((len > (L1+W)) || (len > lstarve)) break;

      B += len*len*len*pop[0][i][number];
    }
  B *= LWCONST/LAKEVOLUME;

  predmort  = ACP*predators/(1+ACP*HCP*B);
  predmort /= LAKEVOLUME;

  for(i=0; i<cohort_no[0]; i++)
    {
      popgrad[0][i][number] = -pop[0][i][number];
      popgrad[0][i][length] =  1.0;

      len = pop[0][i][length];
      if (len > lstarve) continue;

      grazing  += len*len*pop[0][i][number];

      if ((len > LJ) && (len <= lmax))
	newborns += Fr*len*len*pop[0][i][number];
      else if (len > LJ)
	newborns += (Fr - KAPPA*len/LM)*len*len*pop[0][i][number]/(1-KAPPA);

      mort = MUC;
      if ((len > L1) && (len < (L1+W)))	mort += predmort;
      if (len > lcrit) mort += MUS*(len - lcrit);
      if ((len > MINSIZEHARV) && (time >= HARVSTART) && ((time -HARVSTART) < HARVDUR))
	mort += MUHARV;
      
      popgrad[0][i][number] = -mort*pop[0][i][number];
      popgrad[0][i][length] = max(GAMMA*(lmax-len),0.0);

    }
  
  // Assume that newborns always have a positive energy balance and
  // experience a size-independent background mortality
  mort = MUC;
  len  = LB;
  
  if (ofs[0][0][number]) len += ofs[0][0][length]/ofs[0][0][number];

  grazing += len*len*ofs[0][0][number];
  if ((len > L1) && (len < (L1+W))) mort += predmort;

  ofsgrad[0][0][number] = -mort*ofs[0][0][number] + RM*newborns;

  ofsgrad[0][0][length] = -mort*ofs[0][0][length] +
    ofs[0][0][number]*GAMMA*(lmax - LB) - GAMMA*ofs[0][0][length];

  envgrad[0] = 1;
  envgrad[1] = RHO*(K - resource) - IM*Fr*grazing/LAKEVOLUME;
  envgrad[2] = max(PINV, EPS*B*predmort)*LAKEVOLUME - MUP*predators;
  
  return;
}




/*
 *===========================================================================
 *
 *		SPECIFICATION OF BETWEEN COHORT CYCLE DYNAMICS
 *
 *===========================================================================
 */

void	InstantDynamics(double *env, population *pop, population *ofs)
  
{
  double		lstarve;
  register int		i;

  lstarve  = (LM*resource/(RH+resource))/KAPPA;
  
  for(i=0; i<cohort_no[0]; i++)
    {
      if ((pop[0][i][length] > lstarve) ||
	  (pop[0][i][number] < (MIN_SURVIVAL*popIDcard[0][i][IDnbegin]))) pop[0][i][number] = 0.0;
      popIDcard[0][i][IDage] += cohort_limit;
    }

  ofsIDcard[0][0][IDage]    = 0.5*cohort_limit;
  ofsIDcard[0][0][IDnbegin] = ofs[0][0][number];


  return;
}




/*
 *===========================================================================
 *
 *			SPECIFICATION OF OUTPUT VARIABLES
 *
 *===========================================================================
 */

void	DefineOutput(double *env, population *pop, double *output)
  
{
  register int             i;
  double                   Fr, len, lmax, lstarve, lcrit;
  double		   B, predmort;
  double                   num, mass, newborns;

  Fr       = resource/(RH+resource);
  lmax     = LM*Fr;
  lstarve  = lmax/KAPPA;
  lcrit    = lstarve - QS*(lstarve - lmax);
  
  B        = 0.0;

  for(i=cohort_no[0]-1; i>=0; i--)
    {
      len = pop[0][i][length];

      if (len < L1) continue;
      if ((len > (L1+W)) || (len > lstarve)) break;

      B += len*len*len*pop[0][i][number];
    }
  B *= LWCONST/LAKEVOLUME;

  predmort  = ACP*predators/(1+ACP*HCP*B);
  predmort /= LAKEVOLUME;

  output[ 0] = resource;			// Column 2
  output[ 1] = predators;			// Column 3
  
  for(i=0, newborns=0.0; i<cohort_no[0]; i++)
    {
      num  = pop[0][i][number]/LAKEVOLUME;
      len  = pop[0][i][length];
      mass = LWCONST*len*len*len*num;
      
      // Total abundance and mass
      output[ 2] += num;
      output[ 3] += mass;			// Column 5

      // Consumer abundance and mass below vulnerability window
      if (len <= L1)
	{
	  output[ 4] += num;
	  output[ 5] += mass;			// Column 7
	}
      // Consumer abundance and mass in vulnerability window
      else if (len <= (L1+W))
	{
	  output[ 6] += num;
	  output[ 7] += mass;			// Column 9
	}
      // Juvenile consumer abundance and mass above vulnerability window
      else if (len <= LJ)
	{
	  output[ 8] += num;
	  output[ 9] += mass;			// Column 11
	}
      // Adult consumer abundance and mass above vulnerability window
      else
	{
	  output[10] += num;
	  output[11] += mass;			// Column 13

	  if (pop[0][i][length] <= lmax)
	    newborns += RM*Fr*len*len*num;
	  else if (pop[0][i][length] > lmax)
	    newborns += RM*(Fr - KAPPA*len/LM)*len*len*num/(1-KAPPA);
	}
    }

  output[12] = newborns;
  if (output[10]) output[13] = newborns/output[10];
  output[14] = pop[0][0][length];		// Column 16
  output[15] = B*LAKEVOLUME;

  for(i=0; i<cohort_no[0]; i++)
    {
      len = pop[0][i][length];
      popIDcard[0][i][IDgrowth]      = max(GAMMA*(lmax-len),0.0);
      if ((len > LJ) && (len <= lmax))
	popIDcard[0][i][IDfecundity] = RM*Fr*len*len;
      else if (len > LJ)
	popIDcard[0][i][IDfecundity] = RM*(Fr - KAPPA*len/LM)*len*len/(1-KAPPA);
      else
	popIDcard[0][i][IDfecundity] = 0.0;
      if ((len > L1) && (len < (L1+W)))
	popIDcard[0][i][IDpmort]     = predmort;
      else
	popIDcard[0][i][IDpmort]     = 0.0;
      if (len > lcrit)
	popIDcard[0][i][IDsmort]     = MUS*(len - lcrit);
      else
	popIDcard[0][i][IDsmort]     = 0.0;
    }

  return;
}




/*===========================================================================*/
